import 'package:flutter/material.dart';

///Animation types
enum AnimationType { normal, fadeIn }

///Main class, [context] refers to the BuildContext of the app at that moment
///[child] receives the Widget to navigate, [animation] is the type of the animation
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {required this.context,
      required this.child,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false}) {
    switch (animation) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
    }
  }

  ///Normal push of  page
  void _pushPage(Route route) => Navigator.push(context, route);

  ///Push replacement of page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  ///Handler for ormal transition
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => child);

    (replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }

  ///Handle for fade in transition
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) {
        return FadeTransition(
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut)),
          child: child,
        );
      },
    );

    (replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }
}
